﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;

public class Cuffs : MVRScript {

    Transform GetChainEnd( Transform crnt)
    {
        Transform FoundTrans = null;
        //SuperController.LogMessage(crnt.name);
        if (crnt.name == "ChainEnd")
            return crnt;

        for (int i = 0; i < crnt.childCount; i++)
        {
            FoundTrans = GetChainEnd(crnt.GetChild(i));
            if (FoundTrans != null)
			{
                return FoundTrans;
			}
        }

        return FoundTrans;
    }

    protected JSONStorableStringChooser atomJSON;
    protected void SyncAtomChocies()
    {
        List<string> atomChoices = new List<string>();
        atomChoices.Add("None");
        foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
        {
            if ( findChainEnd(SuperController.singleton.GetAtomByUid(atomUID)) != null )
                atomChoices.Add(atomUID);
        }
        atomJSON.choices = atomChoices;
    }

    protected Atom connectedToAtom = null;
    protected void SyncAtom(string atomUID)
    {
        if (atomUID != null)
        {
            connectedToAtom = SuperController.singleton.GetAtomByUid(atomUID);
            connectTo(findChainEnd(connectedToAtom));
        }
    }


    protected JSONStorableStringChooser materialJSON;
    protected void SyncMaterialChoices()
    {
        List<string> matChoices = new List<string>();
        matChoices.Add("None");
        foreach (MeshRenderer mr in containingAtom.GetComponentsInChildren<MeshRenderer>())
        {
            foreach (Material mat in mr.materials)
            {
                matChoices.Add(mat.name);
            }
        }
        materialJSON.choices = matChoices;
    }

    //protected Atom connectedToAtom = null;
    Material selectedMaterial;
    protected void SyncMaterial(string matName)
    {
        selectedMaterial = null;
        if (matName != null)
        {
            foreach (MeshRenderer mr in containingAtom.GetComponentsInChildren<MeshRenderer>())
            {
                foreach (Material mat in mr.materials)
                {
                    if (mat.name == matName)
                    {
                        selectedMaterial = mat;
                        return;
                    }
                }
            }
        }
    }


    private Rigidbody connectedBody = null;

    Rigidbody findChainEnd(Atom connectedAtom )
    {
        if ( connectedAtom == null )
            return null;

        foreach ( Rigidbody rb in connectedAtom.GetComponentsInChildren<Rigidbody>() )
        {
            if (rb.name == "ChainEnd")
                return rb;
        }

        return null;
    }

    void connectTo( Rigidbody toConnect )
    {
        if (connectedToAtom == null || toConnect == null)
            return;

        foreach (Rigidbody rb in containingAtom.rigidbodies)
        {
            if (rb.name == "object")
            {
                ConfigurableJoint fj = rb.gameObject.GetComponent<ConfigurableJoint>();
                if (fj != null)
                {
                    fj.connectedBody = toConnect;
                    connectedBody = toConnect;
                    fj.xMotion = ConfigurableJointMotion.Locked;
                    fj.yMotion = ConfigurableJointMotion.Locked;
                    fj.zMotion = ConfigurableJointMotion.Locked;
                    fj.autoConfigureConnectedAnchor = false;
                    fj.connectedAnchor = new Vector3(0.0f, -0.05f, 0.0f);
                    rb.mass = toConnect.mass;
                }
                else if (fj == null)
                {
                    fj = rb.gameObject.AddComponent<ConfigurableJoint>();
                    fj.connectedBody = toConnect;
                    connectedBody = toConnect;
                    fj.xMotion = ConfigurableJointMotion.Locked;
                    fj.yMotion = ConfigurableJointMotion.Locked;
                    fj.zMotion = ConfigurableJointMotion.Locked;
                    fj.autoConfigureConnectedAnchor = false;
                    fj.connectedAnchor = new Vector3(0.0f, -0.05f, 0.0f);
                    rb.mass = toConnect.mass;
                }
                else
                {
                    SuperController.LogMessage("Hm.. Strange...");
                }
            }
        }
    }


    protected UIDynamicSlider BumbnessSlider;
    protected UIDynamicSlider SmoothnessSlider;

    public override void Init()
    {
        try
        {
            // make atom selector
            atomJSON = new JSONStorableStringChooser("connectedToAtom", SuperController.singleton.GetAtomUIDs(), null, "Connect to Atom", SyncAtom);
            RegisterStringChooser(atomJSON);
            SyncAtomChocies();
            UIDynamicPopup dp = CreateScrollablePopup(atomJSON);
            dp.popupPanelHeight = 1100f;
            // want to always resync the atom choices on opening popup since atoms can be added/removed
            dp.popup.onOpenPopupHandlers += SyncAtomChocies;

            // make material selector
            materialJSON = new JSONStorableStringChooser("SelectedMaterial", SuperController.singleton.GetAtomUIDs(), null, "Selected Material", SyncMaterial);
            RegisterStringChooser(materialJSON);
            SyncMaterialChoices();
            UIDynamicPopup dpmat = CreateScrollablePopup(materialJSON);
            dpmat.popupPanelHeight = 1100f;
            // want to always resync the atom choices on opening popup since atoms can be added/removed
            dpmat.popup.onOpenPopupHandlers += SyncMaterialChoices;

            JSONStorableFloat jfloat = new JSONStorableFloat("Diffuse Bumbness", 0.5f, BumbnessCallback, 0.0f, 1f);
            RegisterFloat(jfloat);
            BumbnessSlider = CreateSlider(jfloat, true);

            // JSONStorableColor example
            HSVColor hsvc = HSVColorPicker.RGBToHSV(1f, 0f, 0f);
            JSONStorableColor jcolor = new JSONStorableColor("Diffuse Color", hsvc, DiffuseColorCallback);
            RegisterColor(jcolor);
            CreateColorPicker(jcolor);

            jfloat = new JSONStorableFloat("Smoothness", 0.5f, SmoothnessCallback, 0.0f, 1f);
            RegisterFloat(jfloat);
            SmoothnessSlider = CreateSlider(jfloat, true);

            hsvc = HSVColorPicker.RGBToHSV(1f, 0f, 0f);
            jcolor = new JSONStorableColor("Specular Color", hsvc, SpecularColorCallback);
            RegisterColor(jcolor);
            CreateColorPicker(jcolor);

        }
        catch (Exception e)
        {
            SuperController.LogError("Exception caught: " + e);
        }
    }


    private FreeControllerV3 ChainEnd;
	// Use this for initialization
	void Start () {

    }

    // Update is called once per frame
    void Update () {
        if (connectedBody == null)
            connectTo(findChainEnd(connectedToAtom));
            
    }


    protected void SmoothnessCallback(float val)
    {
        if (selectedMaterial != null)
        {
            selectedMaterial.SetFloat("_Glossiness", val);
        }
    }

    protected void BumbnessCallback(float val)
    {
        if (selectedMaterial != null)
        {
            selectedMaterial.SetFloat("_BumpScale", val);
        }
    }

    protected void DiffuseColorCallback(JSONStorableColor jcolor)
    {
        if ( selectedMaterial != null )
        {
            selectedMaterial.SetColor("_Color", jcolor.colorPicker.currentColor);
        }
    }

    protected void SpecularColorCallback(JSONStorableColor jcolor)
    {
        if (selectedMaterial != null)
        {
            selectedMaterial.SetColor("_SpecColor", jcolor.colorPicker.currentColor);
        }
    }

}
